@@ -1,5 +1,21 @@
-TopviewTopographicMap <- function(GeneralizedUmatrix,BestMatchingUnits,Cls,ClsColors=NULL,Imx=NULL,Names=NULL, BmSize=6,...) {
-  #author: Tim Schreier, Luis Winckelmann, MCT
+TopviewTopographicMap <- function(GeneralizedUmatrix,BestMatchingUnits,Cls,
+                                  ClsColors=NULL,Imx=NULL,
+                                  ClsNames=NULL,
+                                  BmSize=6,DotLineWidth=2,alpha=1,...) {
+  #author: Tim Schreier, Luis Winckelmann, MCT, QS
+  
+  #CopyCls = Cls
+  #counter = 1
+  #for(j in sort(unique(Cls))){
+  #  CopyCls[which(Cls == j)] = counter
+  #  counter = counter + 1
+  #}
+  #Cls = CopyCls
+  #Cls <- CopyCls
+  #Cls <<- CopyCls
+  
+  
+  
   udim <- dim(GeneralizedUmatrix)
   
   if (!requireNamespace('plotly',quietly = TRUE)) {
@@ -68,7 +84,6 @@ TopviewTopographicMap <- function(GeneralizedUmatrix,BestMatchingUnits,Cls,ClsCo
       )
     )
   }
-  
   if (!is.vector(Cls)) {
     warning('Cls is not a vector. Calling as.vector()')
     Cls = as.vector(Cls)
@@ -83,7 +98,7 @@ TopviewTopographicMap <- function(GeneralizedUmatrix,BestMatchingUnits,Cls,ClsCo
   }
 
   if (length(Cls) != b[1]) {
-    Cls = rep(1, b[1])
+
     warning(
       paste0(
         'Cls has the length ',
@@ -93,6 +108,7 @@ TopviewTopographicMap <- function(GeneralizedUmatrix,BestMatchingUnits,Cls,ClsCo
         '. Plotting without Cls.'
       )
     )
+    Cls = rep(1, b[1])
   }
   
   #Handle Color ----
@@ -133,59 +149,74 @@ TopviewTopographicMap <- function(GeneralizedUmatrix,BestMatchingUnits,Cls,ClsCo
     session = dots[["Session"]]
   
   if (is.null(dots[["main"]]))
-    main = NULL
+    main = "Topographic Map of Generalized U-Matrix"
   else
     main=dots[["main"]]
   
+  if (is.null(dots[["LegendCex"]]))
+    LegendCex = NULL
+  else
+    LegendCex=dots[["LegendCex"]]
+  
+  if (is.null(dots[["MainCex"]]))
+    MainCex = NULL
+  else
+    MainCex=dots[["MainCex"]]
+
+  if (is.null(dots[["NamesOrientation"]]))
+    NamesOrientation = NULL
+  else
+    NamesOrientation=dots[["NamesOrientation"]]
+  
+  if (is.null(dots[["NamesTitle"]]))
+    NamesTitle = NULL
+  else
+    NamesTitle=dots[["NamesTitle"]]
+  
   #Helper Function ----
-  addclass <- function(class,
-                       plotbmus,
-                       plot,
-                       bmu_cols,
-                       MarkerSize) {
-    
+  addclass <- function(class, plotbmus, plot, bmu_cols, MarkerSize, my_counter,
+                       ClsNames, DotLineWidth, alpha){
     inds <- which(Cls == class)
     x = as.numeric(plotbmus[inds, 2])
     y = as.numeric(plotbmus[inds, 1])
+    # Color names to RGBA = RGB + Opacity
+    if(is.character(bmu_cols)){
+      #vecRGBA  = col2rgb(bmu_cols[class], 1)
+      vecRGBA  = col2rgb(bmu_cols[my_counter], alpha)
+      my_color = paste("rgba(", vecRGBA[1], ",", vecRGBA[2], ",",
+                       vecRGBA[3], ",", alpha,")", sep="")
+    }else{
+      my_color = "rgba(80, 80, 80, .8)"
+    }
+    marker = list(size = MarkerSize,
+                  color = my_color,#bmu_cols[class],
+                  line = list(color="rgba(0, 0, 0, .8)", width = DotLineWidth))
+                  #line = list(color="rgba(80, 80, 80, .8)", width = 3))
 
-    plot <- plotly::add_markers(
-      plot, x=x, y=y,
-
-
-    
-      marker = list(
-        size = MarkerSize,
-        color = bmu_cols[class],
-        line = list(color = "rgba(80, 80, 80, .8)",
-                    width = 1)
-      ),
-      name = paste("Cluster", class)
-    )
+    if(is.null(ClsNames)){
+      name = class
+      #standard names
+      plot <- plotly::add_markers(plot, x=x, y=y, marker = marker,
+                                  name = paste("Cluster", name))
+    }else{
+      name = ClsNames[my_counter]
+      #user names
+      plot <- plotly::add_markers(plot, x=x, y=y, marker = marker,
+                                  name = name)
+    }
 
     return(plot)
   }
   
-  PlotlyUmatrix = function(plotdim,
-                           plotumx,
-                           colormap,
-                           Nrlevels2,
-                           plotbmus,
-                           class,
-                           ClsColors,
-                           MarkerSize,
-                           ShinyBinding,
-                           ShinyDimension,
-                           Imx,Cls) {
-    
-
+  
+  PlotlyUmatrix = function(plotdim, plotumx, colormap, Nrlevels2, plotbmus,
+                           class, ClsColors, MarkerSize, ShinyBinding,
+                           ShinyDimension, Imx, Cls, DotLineWidth, alpha){
     # configure filter, so that every bestmatch stays in
- 
-    
     # put Imx on Umatrix and bestmatches if given
     if(!is.null(Imx)){
       if(!is.null(plotbmus)){
         BestMatchesFilter = rep(T,nrow(plotbmus)) # every Bestmatch stays
-        
       }
       plotumx[which(Imx == 1)] = 0
       bigImx = Imx
@@ -196,18 +227,15 @@ TopviewTopographicMap <- function(GeneralizedUmatrix,BestMatchingUnits,Cls,ClsCo
           if(Imx[i,j] == 1){
             #plotumx[i,j] = NA
             if(!is.null(plotbmus)){
-              
               BestMatchesFilter[(plotbmus[,1] == i) & (plotbmus[,2] == j)] = F
             }
-              
           }
         }
       }
       #BestMatchesFilter = rep(T,nrow(plotbmus)) # every Bestmatch stays
       #print(str(BestMatchesFilter))
       if(!is.null(plotbmus)) plotbmus = plotbmus[BestMatchesFilter,]
-
-      #äprint(str(class))
+      #print(str(class))
       #print(class)
       #todo, ohne globale variable loesen
       if(!is.null(Cls)) Cls <<- Cls[BestMatchesFilter]
@@ -226,84 +254,54 @@ TopviewTopographicMap <- function(GeneralizedUmatrix,BestMatchingUnits,Cls,ClsCo
       plotumx <- plotumx[startLine:endLine,startCol:endCol]
       Imx <- Imx[startLine:endLine,startCol:endCol]
       bigImx <- bigImx[startLine:endLine,startCol:endCol]
-      
     }
-   
-
-
-    ax <- list(
-      title = "",
-      zeroline = FALSE,
-      showline = FALSE,
-      #showticklabels = FALSE,
-      showgrid = FALSE
+    ax <- list(title = "", zeroline = FALSE, showline = FALSE, #showticklabels = FALSE,
+               showgrid = FALSE
     )
-    ay <- list(
-      autorange = "reversed",
-      title = "",
-      zeroline = FALSE,
-      showline = FALSE,
-      #showticklabels = FALSE,
-      showgrid = FALSE
-    )
-    
+    ay <- list(autorange = "reversed", title = "", zeroline = FALSE,
+               showline = FALSE, #showticklabels = FALSE,
+               showgrid = FALSE)
     if (isTRUE(ShinyBinding)) {
       width = (0.95 * as.numeric(ShinyDimension[1]))
       height = udim[1] / udim[2] * (width - 80)
       #print(width)
       plt <- plotly::plot_ly(width = width, height = height * 0.75)
-    } else{
+    }else{
       plt <- plotly::plot_ly()
     }
-    plt <- plotly::add_contour(
-      plt,
-      x = 1:plotdim[1],
-      y = 1:plotdim[2],
-      z = plotumx,
-      showscale = FALSE,
-      line = list(color = 'black',
-                  width = 0.5),
-      contours = list(
-        start = 0,
-        end = 1,
-        size = 1 / 15
-      ),
-      # colors = colorRamp(colormap[c(rep(3, 6),
-      #                               seq(
-      #                                 from = 4,
-      #                                 to = length(colormap) - 30,
-      #                                 length.out = ceiling(Nrlevels2 + 1) - 7
-      #                               ),
-      #                               length(colormap))]),
-      colors = colorRamp(colormap[c(  1,2,
-                                      seq(
-                                      from = 3,
-                                      to = length(colormap) - 30,
-                                      length.out = ceiling(Nrlevels2 + 1)- 4
-                                    ),length(colormap),
-                                    length(colormap))]),
-
-
-      name = "UMatrix"
-      # , showscale = FALSE
-    )
-
-    for (class in unique(Cls)) {
-      
-      plt <- addclass(class, plotbmus, plt, ClsColors, MarkerSize)
+    plt <- plotly::add_contour(plt, x = 1:plotdim[1], y = 1:plotdim[2], 
+                               z = plotumx, showscale = FALSE, 
+                               line = list(color = 'black', width = 0.5), 
+                               contours = list(start=0, end=1, size=1/15),
+                               colors = colorRamp(colormap[c(  1,2,
+                                      seq(from=3, to=length(colormap)-30, 
+                                          length.out = ceiling(Nrlevels2+1)-4),
+                                      length(colormap), length(colormap))]),
+                                # colors = colorRamp(colormap[c(rep(3, 6),
+                                #                               seq(
+                                #                                 from = 4,
+                                #                                 to = length(colormap) - 30,
+                                #                                 length.out = ceiling(Nrlevels2 + 1) - 7
+                                #                               ),
+                                #                               length(colormap))]),
+                                name = "UMatrix")
+                                # , showscale = FALSE
+    my_counter = 1
+    ClsColors = ClsColors[order(unique(Cls))]
+    for(class in sort(unique(Cls), decreasing = F)){    # QS: Force stable order in the plot legend
+      plt <- addclass(class, plotbmus, plt, ClsColors, MarkerSize, my_counter,
+                      ClsNames, DotLineWidth, alpha)
+      my_counter = my_counter + 1
     }#end add class
- 
     plt <- plotly::layout(
-      #title <- "Drop plot title here",
-      #bgcolor = "rgb(244, 244, 248)",
       plt,
       xaxis = ax,
       yaxis = ay,
       dragmode = 'lasso',
-      legend = list(orientation = 'h')
+      legend = list(orientation = 'h')#,font = list(size = LegendCex))
       #, showlegend = FALSE
     )
-
+    
     #if (isTRUE(ShinyBinding)) {
     #  requireNamespace('shiny')
     #  shiny::updateSelectInput(session,
@@ -359,7 +357,6 @@ TopviewTopographicMap <- function(GeneralizedUmatrix,BestMatchingUnits,Cls,ClsCo
     plotbmus <- tU$BestMatchingUnits[,2:3] #no key
     plotCls <-  tU$Cls
   } else{
-  
     if(is.null(dots[["ExtendBorders"]])){
       #nothing
     }else{
@@ -375,25 +372,27 @@ TopviewTopographicMap <- function(GeneralizedUmatrix,BestMatchingUnits,Cls,ClsCo
   }
   
   plotdim <- qdim
-
-  plt=PlotlyUmatrix(
-    plotdim,
-    plotumx,
-    colormap,
-    Nrlevels2,
-    plotbmus,
-    class,
-    ClsColors,
-    BmSize,
-    ShinyBinding,
-    ShinyDimension,
-    Imx,
-    Cls
-  )
-  if(is.null(main))
-    plt=plotly::layout(plt,title = "Topographic Map of Generalized U-Matrix")
-  else
-    plt=plotly::layout(plt,title = main)
+  plt=PlotlyUmatrix(plotdim, plotumx, colormap, Nrlevels2, plotbmus, class,
+                    ClsColors, BmSize, ShinyBinding, ShinyDimension, Imx, Cls,
+                    DotLineWidth, alpha)
+  
+  if(!is.null(main))
+    plt=plotly::layout(plt,title = list(text=main))
+  
+  if(!is.null(LegendCex))
+    plt=plotly::layout(plt,legend = list(font = list(size = LegendCex)))
+  
+  if(!is.null(MainCex))
+    plt=plotly::layout(plt,  title=list(font=list(size=MainCex)))
+  
+  if(!is.null(NamesOrientation))
+    plt=plotly::layout(plt,    legend = list(orientation = NamesOrientation))
+  
+  if(!is.null(NamesTitle))
+    plt=plotly::layout(plt,    legend = list(title = list(text=NamesTitle)))
+  
+  if(!is.null(LegendCex)&!is.null(NamesTitle))
+    plt=plotly::layout(plt,    legend = list(title = list(texsize=LegendCex)))
   
   #if (isTRUE(ShinyBinding)) {
   #  PlotR <- plotly::renderPlotly({
@@ -401,7 +400,14 @@ TopviewTopographicMap <- function(GeneralizedUmatrix,BestMatchingUnits,Cls,ClsCo
   #  })
   #  return(list(Rendered=PlotR,single=plt))
   #} else{
-    return(plt)
   #}
- 
-}#end TopviewTopographicMap
+  return(plt)
+}
+
+
+
+
+
+
+
+
